;*******************************************************************************
;*
;*	Titel:	teensylogger (teensylogger_v0_7.asm)          
;*	Datum:	03/02/12	Version: 07 (31/01/14)	
;*	Autor:	WEIGU
;*
;*	Informationen zur Beschaltung:
;*      
;*	Prozessor: ATmega32u4 (Teensy)		Quarzfrequenz:	16MHz
;*	Eingaenge:	8 * Impulseingang fuer SO kWh-Zaehler (PB0-PB7)
;*                      (Externe Pull-Ups, steigende Flanken)
;*			RXD1 an PD2 
;*			SDA an PD1
;*			
;*	Ausgaenge: 	TXD1 an PD3
;*			SCL an PD0	
;*      
;*      Informationen zur Funktionsweise:
;*      
;*      Timer 1 ruft die Timer-Interruptroutine 2x pro Sekunde auf. Diese laesst
;*      die LED blinken und setzt MINFLG nach einer Minute auf 0xFF
;*      (Abspeichern der Werte ein mal pro Minute)
;*      
;*      Bei 2000 Impulsen/kWh koennen pro Phase (32A = 7,36kW) maximal 245
;*      Impulse pro Minute auftreten (2000*7,36/60). Für die Minutenwerte reicht
;*      also 1 Byte. Fuer die Viertestundenwerte werden 2 Byte benoetigt
;*      (max. 3680 Impulse/Viertelstunde).
;*      Fuer die Tageswerte werden auch nur 2 Byte verwendet. Die maximale
;*      Dauerleistung pro Tag betraegt bei 1000Imp/kWh also 65536/1000/24 =
;*      2,7kW (11,8A). Bei 2000 Imp/kWh sogar nur die Haelfte)
;*      
;*      Das externen EEPROM (512 kBit = 64kByte) ist aufgeteilt in 512 Seiten 
;*      (Pages) zu je 128 Byte (1kBit).
;*      
;*      448 Seiten werden fuer die Viertestundenwerte verwendet (Adresse 0x0000
;*      bis 0xDFFF). Dabei werden pro Seite 16 Byte Datuminformation
;*      (dd.mm.yy+Space+hh.MM+Space+Space) und 7 Datensaetze mit je 16 Byte
;*      (8*2 Byte pro Datensatz) abgespeichert. Dies reicht fuer 32 Tage und
;*      16 Stunden.  
;*      
;*      64 Seiten stehen fuer die Tageswerte zur Verfuegung (0xE000-0xFFFF).
;*      Pro Seite werden auch hier 7 Datensaetze + Datumsinfo abgespeichert.
;*      Damit koennen die Daten von mehr als einem Jahre (448 Tage) gespeichert
;*      werden.
;*      
;*      Beide Speicherbereiche laufen als Ringbuffer.
;*      Die momentane Speicheradressen befinden sich im internen EEPROM.
;*      Dieses muss also beim ersten Mal mit den beiden Adressen 0x0000 und
;*      0xE000 programmiert werden.
;*      Fuer den Viertelstundenspeicherbereich auf die interne Epromadresse
;*      0x0000:0x0001 (HByte:LByte). Fuer den Tagesspeicherbereich auf die
;*      interne Epromadresse 0x0002:0x0003 (HByte:LByte) 
;*            
;*			
;*******************************************************************************

;-------------------------------------------------------------------------------
;	Einbinden der controllerspezifischen Definitionsdatei
;-------------------------------------------------------------------------------
.NOLIST				;List-Output ausschalten
.INCLUDE "m32u4def.inc"		;AVR-Definitionsdatei einbinden

;folgende Zeilen fehlen in der Defenitionsdatei!
;Bugreport wurde von ATMEL bestaetigt
.equ    TWAMR   = 0xbd  ; MEMORY MAPPED
.equ    TWCR    = 0xbc  ; MEMORY MAPPED
.equ    TWDR    = 0xbb  ; MEMORY MAPPED
.equ    TWAR    = 0xba  ; MEMORY MAPPED
.equ    TWSR    = 0xb9  ; MEMORY MAPPED
.equ    TWBR    = 0xb8  ; MEMORY MAPPED
; ***** TWI **************************
; TWAMR - TWI (Slave) Address Mask Register
.equ    TWAM0   = 1     ;
.equ    TWAMR0  = TWAM0 ; For compatibility
.equ    TWAM1   = 2     ;
.equ    TWAMR1  = TWAM1 ; For compatibility
.equ    TWAM2   = 3     ;
.equ    TWAMR2  = TWAM2 ; For compatibility
.equ    TWAM3   = 4     ;
.equ    TWAMR3  = TWAM3 ; For compatibility
.equ    TWAM4   = 5     ;
.equ    TWAMR4  = TWAM4 ; For compatibility
.equ    TWAM5   = 6     ;
.equ    TWAMR5  = TWAM5 ; For compatibility
.equ    TWAM6   = 7     ;
.equ    TWAMR6  = TWAM6 ; For compatibility

; TWBR - TWI Bit Rate register
.equ    TWBR0   = 0     ;
;*			SCL an PD0	
.equ    TWBR1   = 1     ;
.equ    TWBR2   = 2     ;
.equ    TWBR3   = 3     ;
.equ    TWBR4   = 4     ;
.equ    TWBR5   = 5     ;
.equ    TWBR6   = 6     ;
.equ    TWBR7   = 7     ;

; TWCR - TWI Control Register
.equ    TWIE    = 0     ; TWI Interrupt Enable
.equ    TWEN    = 2     ; TWI Enable Bit
.equ    TWWC    = 3     ; TWI Write Collition Flag
.equ    TWSTO   = 4     ; TWI Stop Condition Bit
.equ    TWSTA   = 5     ; TWI Start Condition Bit
.equ    TWEA    = 6     ; TWI Enable Acknowledge Bit
.equ    TWINT   = 7     ; TWI Interrupt Flag

; TWSR - TWI Status Register
.equ    TWPS0   = 0     ; TWI Prescaler
.equ    TWPS1   = 1     ; TWI Prescaler
.equ    TWS3    = 3     ; TWI Status
.equ    TWS4    = 4     ; TWI Status
.equ    TWS5    = 5     ; TWI Status
.equ    TWS6    = 6     ; TWI Status
.equ    TWS7    = 7     ; TWI Status

; TWDR - TWI Data register
.equ    TWD0    = 0     ; TWI Data Register Bit 0
.equ    TWD1    = 1     ; TWI Data Register Bit 1
.equ    TWD2    = 2     ; TWI Data Register Bit 2
.equ    TWD3    = 3     ; TWI Data Register Bit 3
.equ    TWD4    = 4     ; TWI Data Register Bit 4
.equ    TWD5    = 5     ; TWI Data Register Bit 5
.equ    TWD6    = 6     ; TWI Data Register Bit 6
.equ    TWD7    = 7     ; TWI Data Register Bit 7

; TWAR - TWI (Slave) Address register
.equ    TWGCE   = 0     ; TWI General Call Recognition Enable Bit
.equ    TWA0    = 1     ; TWI (Slave) Address register Bit 0
.equ    TWA1    = 2     ; TWI (Slave) Address register Bit 1
.equ    TWA2    = 3     ; TWI (Slave) Address register Bit 2
.equ    TWA3    = 4     ; TWI (Slave) Address register Bit 3
.equ    TWA4    = 5     ; TWI (Slave) Address register Bit 4
.equ    TWA5    = 6     ; TWI (Slave) Address register Bit 5
.equ    TWA6    = 7     ; TWI (Slave) Address register Bit 6

.LIST				;List-Output wieder einschalten

;-------------------------------------------------------------------------------
;       Organisation des Datenspeichers (SRAM)
;-------------------------------------------------------------------------------
.DSEG                           ;was ab hier folgt kommt in den SRAM-Speicher
CNTTAB:	.BYTE	8		;Tabelle mit 8 Zaehlern (PCINT0-PCINT7)
DBUFFM:	.BYTE	8		;Zwischenspeicher fuer Daten (Minuten)
DBUFFQ:	.BYTE	16		;Zwischenspeicher fuer Daten (Viertelstunden)
DBUFFD:	.BYTE	16		;Zwischenspeicher fuer Daten (Tag)
EBUFFQ:	.BYTE	128		;Zwischenspeicher EEPROM-Daten (Viertelstunden)
EBUFFD:	.BYTE	128		;Zwischenspeicher EEPROM-Daten (Tag)
PCNTL:	.BYTE	1		;16 Bit Seitenzaehler (page count)
PCNTH:	.BYTE	1		
EEPPOI:	.BYTE	8		;Zwischenspeicher fuer Pointer (Befehl s ...)
MINFLG:	.BYTE	1		;Minutenflag
PCINTF: .BYTE   1               ;PIN Change Int. Flag Byte (Flankenkontrolle)
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Programmspeicher (FLASH)     Programmstart nach RESET ab Adr. 0x0000
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.CSEG                           ;was ab hier folgt kommt in den FLASH-Speicher
.ORG	0x0000			;Programm beginnt an der FLASH-Adresse 0x0000
RESET1:	rjmp	INIT		;springe nach INIT (ueberspringe ISR Vektoren)

;-------------------------------------------------------------------------------
;       Sprungadressen fuer die Interrupts organisieren (ISR VECTORS)
;-------------------------------------------------------------------------------
        ;Vektortabelle (im Flash-Speicher)
.ORG    PCI0addr                ;interner Vektor fuer PCINT
        rjmp    ISRPCS          ;Springe zur ISR des Pin Change Interrupt
.ORG    USB_GENaddr             ;interner Vektor fuer USB General Interrupt
        jmp    ISRGEN           ;Springe zur ISR des GENeral Interrupt
.ORG    USB_COMaddr             ;interner Vektor fuer USB Endpoint/Pipe
                                ;COMmunication Interrupt
        jmp    ISRCOM           ;Springe zur ISR des EP COMmunication Interrupt
.ORG    OC1Aaddr                ;Timer/Counter1 Compare Match A
        rjmp    ISRTC1          ;springe zur ISR Timer1        
.ORG    URXC1addr               ;Vektor fuer Interrupt RX (EIA232 Empfang)
        jmp	ISRRX
.ORG    UDRE1addr               ;Vektor fuer Interrupt UDRE (EIA232 Senden)
        jmp	ISRTX
        
;-------------------------------------------------------------------------------
;	Initialisierungen und eigene Definitionen
;-------------------------------------------------------------------------------
.ORG    INT_VECTORS_SIZE        ;Platz fuer ISR Vektoren lassen
INIT:
.DEF    Zero = r15              ;Register 1 wird zum Rechnen benoetigt
        clr     r15             ;und mit Null belegt
.DEF    Tmp1 = r16              ;Register 16 dient als erster Zwischenspeicher
.DEF    Tmp2 = r17              ;Register 17 dient als zweiter Zwischenspeicher
.DEF    Cnt1 = r18              ;Register 18 dient als Universal-Zaehler
;ISR !! Achtung globale Variablen !!     
.DEF    SecCnt = r19            ;Register 19 dient als Sekundenzaehler fuer die
.DEF    Q7Cnt = r20             ;Register 20 dient als Zaehler 
.DEF    D7Cnt = r21             ;Register 21 dient als Zaehler 
.DEF    PPointL = r22           ;Reg. 23:22 dient als Seitenzeiger
.DEF    PPointH = r23           ;(page pointer) 
.DEF    WL = r24                ;Register 24 und 25 dienen als universelles
.DEF    WH = r25                ;Doppelregister W und zur Parameteruebergabe

	;Stapel initialisieren (fuer Unterprogramme bzw. Interrupts)
	ldi	Tmp1,HIGH(RAMEND)	;RAMEND (SRAM) ist in der Definitions-
	out	SPH,Tmp1		;datei festgelegt	
	ldi	Tmp1,LOW(RAMEND)
	out	SPL,Tmp1
	
	;CLKPCE = 1 Wirkung des Fusebit CKDIV
        ldi     r16,0x80        ;CLKPCE = 1 Wirkung des Fusebit CKDIV
        sts     CLKPR,r16       ;ueberschreiben
        clr     r16             ;CLKPR = 0        
        sts     CLKPR,r16
        
        ;Timer 1 initialisieren
        ;Interrupt zwei mal pro Sekunde (2Hz)
        ;Vorteiler 256, Zaehlschritte 31250-1 ergeben bei 16 MHz T = 1/2 Sekunde
        clr     Tmp1
        sts     TCCR1A,Tmp1
        ldi     Tmp1,0b00001100 ;WGM12 = 1 Teiler = 256
        sts     TCCR1B,Tmp1
        ldi     Tmp1,HIGH(31249)
        ldi     Tmp2,LOW(31249)
        sts     OCR1AH,Tmp1     ;Achtung High zuerst!
        sts     OCR1AL,Tmp2
        ldi     Tmp1,0x02       ;OCI1A Interrupt erlauben
        sts     TIMSK1,Tmp1

        ;Pin Change Interrupts PCINT0-PCINT7 (PB0-PB7) initialisieren
        out	DDRB,Zero	;alle Eingang
        ldi	Tmp1,0x01	;erlaube Pin Change Interrupt
        sts	PCICR,Tmp1
        ser	Tmp1		;Schalte alle 8 Interrupts frei PCINT0-PCINT7
        sts	PCMSK0,Tmp1	;(PB0-PB7)
        
        ;I2C Bus  und EIA232 initialisieren
        rcall   I2CINI
        rcall   E232INI

        ;Flag loeschen
        sts	MINFLG,Zero	;Minuten-FLAG zuruecksetzen
        
        ;Zaehler loeschen
        clr	SecCnt
        clr	Q7Cnt
        clr	D7Cnt
        
        ;PCINT Zaehler und Buffer loeschen
        ldi	XL,LOW(CNTTAB)
        ldi	XH,HIGH(CNTTAB)
        ldi	Cnt1,8+8+16+16
ILOOP1: st	X+,Zero
        dec	Cnt1
        brne	ILOOP1

        rcall WDT_ON		;Watchdog einschalten


        ;Ausgaenge setzen
        sbi	DDRD,6		;Teensy LED
        
        call   USBINI          ;befindet sich in der USB-Bibliothek
        ;Interrupts global erlauben
        ;sei                     befindet sich bereits in USBINI

;-------------------------------------------------------------------------------
;	Hauptprogramm
;-------------------------------------------------------------------------------
MAIN:	sbi	PORTD,6		;LED on and greetings (EIA232)
	call	GREET
	rcall   W1ms
        rcall   RTCRCV		;RTC 2* lesen        
        lds     Tmp1,RTCTAB+0	;Falls Uhr nicht laeuft, dann neu starten
        rcall	W1s
        rcall	W1ms
        rcall   RTCRCV		       
        lds     Tmp2,RTCTAB+0        
        cp	Tmp1,Tmp2
        brne	MAIN1
        ;RTC starten 01.01.12 22:33:00
        ldi     Tmp1,0x00	;Sekunden
        sts     RTCTAB+0,Tmp1
        ldi     Tmp1,0x33	;Minuten
        sts     RTCTAB+1,Tmp1
        ldi     Tmp1,0x22	;Stunden
        sts     RTCTAB+2,Tmp1
        ldi     Tmp1,0x07	;Wochentag (ab Montag)
        sts     RTCTAB+3,Tmp1
        ldi     Tmp1,0x01	;Tag
        sts     RTCTAB+4,Tmp1
        ldi     Tmp1,0x01	;Monat
        sts     RTCTAB+5,Tmp1
        ldi     Tmp1,0x12	;Jahr
        sts     RTCTAB+6,Tmp1
        ldi     Tmp1,0x10	;Control Byte SQW ein (1Hz)
        sts     RTCTAB+7,Tmp1          
        rcall   RTCSND
        rcall	W100ms
        
        ;analyse commands
MAIN1:  lds     Tmp1,RX232FLAG
        cpi     Tmp1,0xFF       ;command received?
        brne    MAIN2

        rcall   COMAN           ;analyse the command and react 
        sts     RX232FLAG,Zero  ;reset flag
        ldi     Cnt1,19         ;clear the buffer
        ldi     XL,LOW(RX232DATA)       
        ldi     XH,HIGH(RX232DATA)
MAIN1L: st      X+,Zero
        dec     Cnt1
        brne    MAIN1L

        ;analyse the minute flag
MAIN2:  lds     Tmp1,MINFLG
        cpi     Tmp1,0xFF       ;jmp MAIN if minute flag (Timer ISR) not set
        breq    MAIN3
        rjmp    MAINR

        ;the following code is executed only once a minute
MAIN3:  sts	MINFLG,Zero	;reset minute flag

	lds	Tmp1,CNTTAB+0	;Momentanwert in Minutenbuffer retten
	sts	DBUFFM+0,Tmp1
	lds	Tmp1,CNTTAB+1
	sts	DBUFFM+1,Tmp1
	lds	Tmp1,CNTTAB+2
	sts	DBUFFM+2,Tmp1
	lds	Tmp1,CNTTAB+3
	sts	DBUFFM+3,Tmp1
	lds	Tmp1,CNTTAB+4
	sts	DBUFFM+4,Tmp1
	lds	Tmp1,CNTTAB+5
	sts	DBUFFM+5,Tmp1
	lds	Tmp1,CNTTAB+6
	sts	DBUFFM+6,Tmp1
	lds	Tmp1,CNTTAB+7
	sts	DBUFFM+7,Tmp1

       	;PCINT Zaehlertabelle (CNTTAB) loeschen
        ldi	XL,LOW(CNTTAB)	
        ldi	XH,HIGH(CNTTAB)
        ldi	Cnt1,8
M3_L1:  st	X+,Zero
        dec	Cnt1
        brne	M3_L1

	;add minute value to quarter data
	ldi	XL,LOW(DBUFFM)	
        ldi	XH,HIGH(DBUFFM)
       	ldi	YL,LOW(DBUFFQ)	
        ldi	YH,HIGH(DBUFFQ)
        ldi	Cnt1,8
M3_L2:  ld	r2,X+
	ld	r1,Y
	ldd	r0,Y+1
	add	r0,r2
	adc	r1,Zero
	st	Y+,r1
	st	Y+,r0
        dec	Cnt1
        brne	M3_L2

	;add minute value to day data (max. 65kWh!)
	ldi	XL,LOW(DBUFFM)	
        ldi	XH,HIGH(DBUFFM)
       	ldi	YL,LOW(DBUFFD)	
        ldi	YH,HIGH(DBUFFD)
        ldi	Cnt1,8
M3_L3:  ld	r2,X+
	ld	r1,Y
	ldd	r0,Y+1
	add	r0,r2
	adc	r1,Zero
	st	Y+,r1
	st	Y+,r0
        dec	Cnt1
        brne	M3_L3
        
        rcall   RTCRCV          ;get date and time
        rcall   RTCASC          ;RTC data (BCD) to ASCII

	;save the quarter data
	;date and time (16 Byte, fist data set) + 7 data sets per page
	;dd.mm.yy+Space+hh.MM+Space+Space + 7*(8*2)byte=112 Byte (HByte first)
	lds	Tmp1,RTCTAB+1
	cpi	Tmp1,0x00
	breq	M3_QDT
	cpi	Tmp1,0x15
	breq	M3_QDT
	cpi	Tmp1,0x30
	breq	M3_QDT
	cpi	Tmp1,0x45
	breq	M3_QDT
	rjmp	M3_DAY	
M3_QDT:	;save date and time
        cpi	Q7Cnt,0	        ;Q7CNT = 0 first data set -> save date and time
	brne	M3_QD
   	ldi	XL,LOW(RTCASCT1) ;save date/time to buffer
        ldi	XH,HIGH(RTCASCT1)
        ldi	YL,LOW(EBUFFQ)	
        ldi	YH,HIGH(EBUFFQ)        
        ldi	Cnt1,14
M3_QL1: ld	Tmp1,X+
	st	Y+,Tmp1
        dec	Cnt1
        brne	M3_QL1
	ldi	Tmp1,' '
	st	Y+,Tmp1
	st	Y,Tmp1
	inc	Q7Cnt
M3_QD:	;save 16 data bytes
        ldi	XL,LOW(DBUFFQ)	        
        ldi	XH,HIGH(DBUFFQ)
	ldi	YL,LOW(EBUFFQ)	
        ldi	YH,HIGH(EBUFFQ)
        ldi	Tmp1,16	        ;calc offset for EBUFFQ
        mul	Tmp1,Q7Cnt
        add	YL,r0
        adc	YH,Zero
        ldi	Cnt1,16
M3_QL2: ld	Tmp1,X+
	st	Y+,Tmp1
        dec	Cnt1
        brne	M3_QL2
        inc     Q7Cnt
        ;clear the DBUFFQ-table
        ldi	XL,LOW(DBUFFQ)	
        ldi	XH,HIGH(DBUFFQ)
        ldi	Cnt1,16
M3_QL3: st	X+,Zero
        dec	Cnt1
        brne	M3_QL3
        ;save page to EEPROM (every quarter hour!)
        ldi	XL,LOW(EBUFFQ)		
        ldi	XH,HIGH(EBUFFQ)
	ldi	YL,LOW(E24PAGE)	
        ldi	YH,HIGH(E24PAGE)        
        ldi	Cnt1,128
M3_QL4: ld	Tmp1,X+
	st	Y+,Tmp1
        dec	Cnt1
        brne	M3_QL4
        ;get current q pointer (result in EEPDAT)
        ldi	r25,0x00
        ldi     r24,0x00        ;HByte
        rcall	EEPRRD
        lds     Tmp1,EEPDAT
        sts     E24ADH,Tmp1
        ldi     r24,0x01        ;LByte
        rcall	EEPRRD
        lds     Tmp1,EEPDAT
        sts     E24ADL,Tmp1
	rcall	E24SP      	;write data
	rcall   W10ms           ;wait to be sure the saving ends
	;reset Q7Counter
        cpi	Q7Cnt,8         ;if all 7 data sets written,
        brne	M3_DAY
        clr	Q7Cnt		;reset Q7Cnt
        ;ringbuffer = ringbuffer + 128
        ldi     Cnt1,128          
        lds     Tmp2,E24ADH
        lds     Tmp1,E24ADL              
        add     Tmp1,Cnt1
        adc     Tmp2,Zero        
        cpi     Tmp2,0xE0       ;if ringbuffer overflow back to 0x0000 
        brne    M3_QD1  
        clr     Tmp2
        clr     Tmp1
M3_QD1: ldi     r25,0x00        ;write HByte
        ldi     r24,0x00                        
        sts     EEPDAT,Tmp2
        rcall   EEPRWR
        ldi     r24,0x01        ;write LByte
        sts     EEPDAT,Tmp1
        rcall   EEPRWR
        
        ;save the day data
        ;date and time (16 Byte, fist data set) + 7 data sets per page
        ;dd.mm.yy+Space+hh.MM+Space+Space + 7*(8*2)byte=112 Byte (HByte first)
M3_DAY: lds	Tmp1,RTCTAB+2
	cpi	Tmp1,0x23
	brne	M3_DR
	lds	Tmp1,RTCTAB+1
	cpi	Tmp1,0x59
	breq	M3_DDT
M3_DR:	rjmp	MAINR
M3_DDT:	;save date and time
	cpi	D7Cnt,0		;Q7CNT = 0 first data set -> save date and time
	brne	M3_DD
   	ldi	XL,LOW(RTCASCT1)	
        ldi	XH,HIGH(RTCASCT1)
        ldi	YL,LOW(EBUFFD)	
        ldi	YH,HIGH(EBUFFD)        
        ldi	Cnt1,14
M3_DL1: ld	Tmp1,X+
	st	Y+,Tmp1
        dec	Cnt1
        brne	M3_DL1
	ldi	Tmp1,' '
	st	Y+,Tmp1
	st	Y,Tmp1
        inc     D7Cnt
M3_DD:	;save 16 data bytes
        ldi	XL,LOW(DBUFFD)	
        ldi	XH,HIGH(DBUFFD)
	ldi	YL,LOW(EBUFFD)	
        ldi	YH,HIGH(EBUFFD)
        ldi	Tmp1,16		;calc offset for EBUFFD
        mul	Tmp1,D7Cnt
        add	YL,r0
        adc	YH,Zero
        ldi	Cnt1,16
M3_DL2: ld	Tmp1,X+
	st	Y+,Tmp1
        dec	Cnt1
        brne	M3_DL2
        inc     D7Cnt
        ;clear the DBUFFD-table
        ldi	XL,LOW(DBUFFD)	
        ldi	XH,HIGH(DBUFFD)
        ldi	Cnt1,16
M3_DL3: st	X+,Zero
        dec	Cnt1
        brne	M3_DL3
        ;save page to EEPROM (every day!)        
        ldi	XL,LOW(EBUFFD)
        ldi	XH,HIGH(EBUFFD)
	ldi	YL,LOW(E24PAGE)	
        ldi	YH,HIGH(E24PAGE)        
        ldi	Cnt1,128
M3_DL4: ld	Tmp1,X+
	st	Y+,Tmp1
        dec	Cnt1
        brne	M3_DL4
        ;get current d pointer (result in EEPDAT)
        ldi	r25,0x00                ;HByte
        ldi	r24,0x02		
        rcall	EEPRRD
        lds     Tmp1,EEPDAT
        sts     E24ADH,Tmp1
        ldi	r24,0x03                ;LByte
        rcall	EEPRRD
        lds     Tmp1,EEPDAT
        sts     E24ADL,Tmp1
	rcall	E24SP        		;write data
	rcall   W10ms                   ;wait to be sure the saving ends
        ;reset Q7Counter
        cpi     D7Cnt,8
        brne    MAINR
        clr     D7Cnt                 
        ;ringbuffer = ringbuffer + 128
        ldi     Cnt1,128                 ;Ringbuffer um 128 Byte erhoehen
        lds     Tmp2,E24ADH
        lds     Tmp1,E24ADL              
        add     Tmp1,Cnt1
        adc     Tmp2,Zero        
        cpi     Tmp2,0x00               ;if ringbuffer overflow back to 0xE000 
        brne    M3_DD1
        ldi     Tmp2,0xE0  
        clr     Tmp1
M3_DD1: ldi     r25,0x00                ;HByte schreiben                        
        ldi     r24,0x02                        
        sts     EEPDAT,Tmp2
        rcall   EEPRWR
        ldi     r24,0x03
        sts     EEPDAT,Tmp1
        rcall   EEPRWR

MAINR:  ;wdr                            ;Watchdog ruecksetzen
        rjmp    MAIN1
;-------------------------------------------------------------------------------
;	Unterprogramme und Interrupt-Behandlungsroutinen
;-------------------------------------------------------------------------------
;Interruptroutine wird zwei mal pro Sekunde durch Timer1 aufgerufen
;Laesst LED (PD6) des Teensy im Sekundentakt blinken und setzt MINFLG nach einer
;Minute auf 0xFF (Abspeichern der Werte ein mal pro Minute)
;SecCnt ist globale Variable!
ISRTC1: push    r16
        in      r16,SREG
        push    r16
        
        sbic	PIND,6
        rjmp	ISRTC1C
        sbi	PORTD,6
        inc	SecCnt
        cpi	SecCnt,60
        brne	ISRTC1R
        clr	SecCnt
        ldi	r16,0xFF
	sts	MINFLG,r16
        rjmp	ISRTC1R
ISRTC1C: cbi	PORTD,6	
     
ISRTC1R: wdr
        pop     r16
        out     SREG,r16
        pop     r16
        reti            
        
;-------------------------------------------------------------------------------
;Pin Change Interrupt-Routine
;Zaehlt positive (steigende) Flanken!
ISRPCS: push    r16
        in      r16,SREG
        push    r16
        
        sbis    PINB,0
        rjmp    ISRPC01         ;falls 0 dann loesche PCINTF0
        lds     r16,PCINTF
        andi    r16,0x01        ;PINB = 1 PCINTF0 auch 1
        brne    ISRPC1          ;falls ja dann weiter
        lds     r16,CNTTAB+0    ;Flanke, erhoehe Zaehler!
        inc     r16
        sts     CNTTAB+0,r16
        lds     r16,PCINTF      ;setze PCINTF0
        ori     r16,0x01
        sts     PCINTF,r16
        rjmp    ISRPCR          ;raus
ISRPC01: lds    r16,PCINTF      ;PCINTF0 loeschen
        andi    r16,0xFE
        sts     PCINTF,r16
        
ISRPC1: sbis    PINB,1
        rjmp    ISRPC11         ;falls 0 dann loesche PCINTF1
        lds     r16,PCINTF
        andi    r16,0x02        ;PINB1 = 1 PCINTF1 auch 1
        brne    ISRPC2          ;falls ja dann weiter
        lds     r16,CNTTAB+1    ;Flanke, erhoehe Zaehler!
        inc     r16
        sts     CNTTAB+1,r16
        lds     r16,PCINTF      ;setze PCINTF1
        ori     r16,0x02
        sts     PCINTF,r16
        rjmp    ISRPCR          ;raus
ISRPC11: lds    r16,PCINTF      ;PCINTF1 loeschen
        andi    r16,0xFD
        sts     PCINTF,r16

ISRPC2: sbis    PINB,2
        rjmp    ISRPC21         ;falls 0 dann loesche PCINTF2
        lds     r16,PCINTF
        andi    r16,0x04        ;PINB2 = 1 PCINTF2 auch 1
        brne    ISRPC3          ;falls ja dann weiter
        lds     r16,CNTTAB+2    ;Flanke, erhoehe Zaehler!
        inc     r16
        sts     CNTTAB+2,r16
        lds     r16,PCINTF      ;setze PCINTF2
        ori     r16,0x04
        sts     PCINTF,r16
        rjmp    ISRPCR          ;raus
ISRPC21: lds    r16,PCINTF      ;PCINTF2 loeschen
        andi    r16,0xFB
        sts     PCINTF,r16
        
ISRPC3: sbis    PINB,3
        rjmp    ISRPC31         ;falls 0 dann loesche PCINTF3
        lds     r16,PCINTF
        andi    r16,0x08        ;PINB3 = 1 PCINTF3 auch 1
        brne    ISRPC4          ;falls ja dann weiter
        lds     r16,CNTTAB+3    ;Flanke, erhoehe Zaehler!
        inc     r16
        sts     CNTTAB+3,r16
        lds     r16,PCINTF      ;setze PCINTF3
        ori     r16,0x08
        sts     PCINTF,r16
        rjmp    ISRPCR          ;raus
ISRPC31: lds    r16,PCINTF      ;PCINTF3 loeschen
        andi    r16,0xF7
        sts     PCINTF,r16

ISRPC4: sbis    PINB,4
        rjmp    ISRPC41         ;falls 0 dann loesche PCINTF4
        lds     r16,PCINTF
        andi    r16,0x10        ;PINB4 = 1 PCINTF4 auch 1
        brne    ISRPC5          ;falls ja dann weiter
        lds     r16,CNTTAB+4    ;Flanke, erhoehe Zaehler!
        inc     r16
        sts     CNTTAB+4,r16
        lds     r16,PCINTF      ;setze PCINTF4
        ori     r16,0x10
        sts     PCINTF,r16
        rjmp    ISRPCR          ;raus
ISRPC41: lds    r16,PCINTF      ;PCINTF4 loeschen
        andi    r16,0xEF
        sts     PCINTF,r16

ISRPC5: sbis    PINB,5
        rjmp    ISRPC51         ;falls 0 dann loesche PCINTF5
        lds     r16,PCINTF
        andi    r16,0x20        ;PINB5 = 1 PCINTF5 auch 1
        brne    ISRPC6          ;falls ja dann weiter
        lds     r16,CNTTAB+5    ;Flanke, erhoehe Zaehler!
        inc     r16
        sts     CNTTAB+5,r16
        lds     r16,PCINTF      ;setze PCINTF5
        ori     r16,0x20
        sts     PCINTF,r16
        rjmp    ISRPCR          ;raus
ISRPC51: lds    r16,PCINTF      ;PCINTF5 loeschen
        andi    r16,0xDF
        sts     PCINTF,r16

ISRPC6: sbis    PINB,6
        rjmp    ISRPC61         ;falls 0 dann loesche PCINTF6
        lds     r16,PCINTF
        andi    r16,0x40        ;PINB6 = 1 PCINTF6 auch 1
        brne    ISRPC7          ;falls ja dann weiter
        lds     r16,CNTTAB+6    ;Flanke, erhoehe Zaehler!
        inc     r16
        sts     CNTTAB+6,r16
        lds     r16,PCINTF      ;setze PCINTF6
        ori     r16,0x40
        sts     PCINTF,r16
        rjmp    ISRPCR          ;raus
ISRPC61: lds    r16,PCINTF      ;PCINTF6 loeschen
        andi    r16,0xBF
        sts     PCINTF,r16

ISRPC7: sbis    PINB,7
        rjmp    ISRPC71         ;falls 0 dann loesche PCINTF7
        lds     r16,PCINTF
        andi    r16,0x80        ;PINB7 = 1 PCINTF7 auch 1
        brne    ISRPCR          ;falls ja dann weiter
        lds     r16,CNTTAB+7    ;Flanke, erhoehe Zaehler!
        inc     r16
        sts     CNTTAB+7,r16
        lds     r16,PCINTF      ;setze PCINTF7
        ori     r16,0x80
        sts     PCINTF,r16
        rjmp    ISRPCR          ;raus
ISRPC71: lds    r16,PCINTF      ;PCINTF7 loeschen
        andi    r16,0x7F
        sts     PCINTF,r16

ISRPCR: pop     r16
        out     SREG,r16
        pop     r16
        reti   
        

        
; include libraries
.INCLUDE "lib/SR_TIME_16M.asm"              ;time
.INCLUDE "lib/SR_MATH.asm"                  ;math
.INCLUDE "lib/SR_I2C.asm"                   ;I2C
.INCLUDE "lib/SR_RTC_DS1307.asm"            ;RTC
.INCLUDE "lib/SR_EEPROM.asm" 		    ;EEPROM (internal)
.INCLUDE "lib/SR_EEPROM_24LC512.asm"        ;EEPROM (external)
.INCLUDE "lib/SR_WATCHDOG.asm"              ;watchdog
.INCLUDE "lib/SR_EIA232.asm"                ;EIA232
.INCLUDE "lib/SR_COMMANDS.asm"              ;analyse commands
.INCLUDE "lib/SR_USB_HID_V1_2.asm"          ;USB

GREET:  push    r16
        ldi     r16,HIGH(GREETT*2)      ;initialize pointer
        sts     TX232P+0,r16
        ldi     r16,LOW(GREETT*2)       
        sts     TX232P+1,r16
        input   r16,UCSRB               ;enable UDRIE
        ori     r16,0x20         
        output  UCSRB,r16    
        pop     r16
        ret

GREETT: .DB     "teensylogger v0.7",0x0D,0x0D,0    ;!! even number of bytes per line !!

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.EXIT				;Ende des Quelltextes
